==> jbossdm-plugin-3.8.1-sources.jar: com/xebialabs/deployit/plugin/jbossdm/step/BaseStep.java <==
package com.xebialabs.deployit.plugin.jbossdm.step;

import java.io.*;
import java.net.URL;
import java.util.List;
import java.util.Map;

import javax.script.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.base.Charsets;
import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import com.google.common.collect.Iterables;
import com.google.common.io.ByteStreams;
import com.google.common.io.Resources;

import com.xebialabs.deployit.plugin.api.flow.ExecutionContext;
import com.xebialabs.deployit.plugin.api.flow.Step;
import com.xebialabs.deployit.plugin.api.flow.StepExitCode;
import com.xebialabs.deployit.plugin.jbossdm.container.CliManagedContainer;
import com.xebialabs.deployit.plugin.jbossdm.exception.CliScriptException;
import com.xebialabs.overthere.OverthereConnection;
import com.xebialabs.overthere.OverthereFile;
import com.xebialabs.overthere.RuntimeIOException;
import com.xebialabs.overthere.local.LocalConnection;
import com.xebialabs.overthere.local.LocalFile;
import com.xebialabs.overthere.util.OverthereUtils;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.base.Strings.emptyToNull;
import static com.google.common.collect.Lists.newArrayList;
import static com.google.common.io.Closeables.closeQuietly;

public abstract class BaseStep implements Step {

    public static final String JYTHON_SCRIPT_ENGINE = "jython";

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    private final String script;
    private final int order;
    private final Map<String, Object> pythonContext;
    private String description;
    private CliManagedContainer container;
    private List<String> additionalLibraries = newArrayList();

    private String remoteWorkingDirPath;
    private boolean retainRemoteWorkingDirOnCompletion;

    private transient OverthereConnection localConn;
    private transient OverthereConnection remoteConn;
    private transient ExecutionContext ctx;
    private transient OverthereFile remoteWorkingDir;


    protected BaseStep(String script, int order, Map<String, Object> pythonContext, String description, CliManagedContainer container) {
        this.script = script;
        this.order = order;
        this.pythonContext = pythonContext;
        this.description = description;
        this.container = container;

    }

    public int getOrder() {
        return order;
    }

    @Override
    public String getDescription() {
        return description;
    }

    public StepExitCode handleExecute(ExecutionContext ctx) throws Exception {
        try {
            this.ctx = ctx;
            ScriptEngine scriptEngine = loadScriptEngine();
            executeScript(scriptEngine);
            return StepExitCode.SUCCESS;
        } finally {
            disconnect();
        }
    }

    protected void executeScript(ScriptEngine scriptEngine) throws IOException {
        Bindings bindings = createBindings(pythonContext);
        bindings.put("step", this);
        loadLibraryScriptsAndEval(script, scriptEngine, bindings);
    }

    protected void loadLibraryScriptsAndEval(String scriptPath, ScriptEngine scriptEngine, Bindings localBindings) throws IOException {
        String script = Resources.toString(Resources.getResource(scriptPath), Charsets.UTF_8);
        //need to do this in order to work around a bug in the python script engine, which does not use the bindings
        //to compile the script when using the method scriptEngine.eval(String,Bindings).
        Bindings origEngineBindings = scriptEngine.getBindings(ScriptContext.ENGINE_SCOPE);
        try {
            Bindings engineAndLocalScope = new SimpleBindings();
            engineAndLocalScope.putAll(origEngineBindings);
            engineAndLocalScope.putAll(localBindings);
            scriptEngine.setBindings(engineAndLocalScope, ScriptContext.ENGINE_SCOPE);
            loadLibraryScripts(additionalLibraries, scriptEngine);
            logger.debug("Executing script " + scriptPath);
            if (logger.isTraceEnabled()) {
                logger.trace(script);
            }
            scriptEngine.eval(script);
        } catch (ScriptException e) {
            throwCliScriptException(scriptPath, e);
        } finally {
            scriptEngine.setBindings(origEngineBindings, ScriptContext.ENGINE_SCOPE);
        }
    }

    protected ScriptEngine loadScriptEngine() throws IOException {
        ScriptEngine scriptEngine = new ScriptEngineManager().getEngineByName(JYTHON_SCRIPT_ENGINE);
        checkNotNull(scriptEngine, "Jython Script Engine cannot be initialized. Make sure jython jars are on the class path.");
        loadLibraryScripts(container.getManagingContainer().getLibraries(), scriptEngine);
        return scriptEngine;
    }

    protected void loadLibraryScripts(List<String> libs, ScriptEngine scriptEngine) throws IOException {
        if (!libs.isEmpty()) {
            for (String library : libs) {
                String script = Resources.toString(Resources.getResource(library), Charsets.UTF_8);
                checkNotNull(script, "Library {} cannot be found on class path.", library);
                try {
                    scriptEngine.eval(script);
                } catch (ScriptException e) {
                    throwCliScriptException(library, e);
                }
            }
        }
    }

    protected void throwCliScriptException(String script, ScriptException e) {
        throw new CliScriptException(script, e);
    }

    protected Bindings createBindings(Map<String, Object> variables) {
        Bindings bindings = new SimpleBindings();
        bindings.putAll(variables);
        return bindings;
    }

    public Object executeCliCommand(String cmd) {
        return container.getManagingContainer().execute(ctx, cmd);
    }


    public ExecutionContext getCtx() {
        return ctx;
    }

    public void setAdditionalLibraries(List<String> additionalLibraries) {
        this.additionalLibraries = additionalLibraries;
    }

    protected List<String> getAdditionalLibraries() {
        return additionalLibraries;
    }

    public CliManagedContainer getContainer() {
        return container;
    }

    public OverthereFile getRemoteWorkingDirectory() {
        if (remoteWorkingDir == null) {
            OverthereFile tempDir;
            if (Strings.isNullOrEmpty(getRemoteWorkingDirPath())) {
                tempDir = getRemoteConnection().getTempFile("jbossdm_plugin", ".tmp");
            } else {
                tempDir = getRemoteConnection().getFile(getRemoteWorkingDirPath());
            }
            tempDir.mkdir();
            remoteWorkingDir = tempDir;
        }
        return remoteWorkingDir;
    }


    public OverthereConnection getLocalConnection() {
        if (localConn == null) {
            localConn = LocalConnection.getLocalConnection();
        }
        return localConn;
    }

    public OverthereConnection getRemoteConnection() {
        if (remoteConn == null) {
            remoteConn = container.getManagingContainer().getHost().getConnection();
        }
        return remoteConn;
    }

    protected void disconnect() {
        if (localConn != null)
            closeQuietly(localConn);

        if (!Strings.isNullOrEmpty(getRemoteWorkingDirPath()) && !isRetainRemoteWorkingDirOnCompletion()) {
            getRemoteWorkingDirectory().deleteRecursively();
        }

        if (remoteConn != null)
            closeQuietly(remoteConn);

        remoteWorkingDir = null;
        localConn = null;
        remoteConn = null;
    }

    public String getRemoteWorkingDirPath() {
        return remoteWorkingDirPath;
    }

    public void setRemoteWorkingDirPath(String remoteWorkingDirPath) {
        this.remoteWorkingDirPath = remoteWorkingDirPath;
    }

    public boolean isRetainRemoteWorkingDirOnCompletion() {
        return retainRemoteWorkingDirOnCompletion;
    }

    public void setRetainRemoteWorkingDirOnCompletion(boolean deleteWorkingDirOnCompletion) {
        this.retainRemoteWorkingDirOnCompletion = deleteWorkingDirOnCompletion;
    }

    public OverthereFile uploadToWorkingDirectory(String content, String fileName) {
        getCtx().logOutput("Uploading file " + fileName + " to working directory.");
        OverthereFile target = getRemoteWorkingDirectory().getFile(fileName);
        OverthereUtils.write(content.getBytes(), target);
        return target;
    }

    public OverthereFile uploadToWorkingDirectory(File content, String fileName) {
        return uploadToWorkingDirectory(LocalFile.valueOf(content), fileName);
    }

    public OverthereFile uploadToWorkingDirectory(OverthereFile content, String fileName) {
        String fileType = content.isDirectory() ? "directory" : "file";
        getCtx().logOutput("Uploading " + fileType + " " + fileName + " to working directory.");
        OverthereFile target = getRemoteWorkingDirectory().getFile(fileName);
        content.copyTo(target);
        return target;
    }

    public OverthereFile uploadToWorkingDirectory(URL content, String fileName) {
        getCtx().logOutput("Uploading file " + fileName + " to working directory.");
        OverthereFile target = getRemoteWorkingDirectory().getFile(fileName);
        OutputStream out = target.getOutputStream();
        try {
            Resources.copy(content, out);
        } catch (IOException e) {
            throw new RuntimeIOException(e);
        } finally {
            closeQuietly(out);
        }
        return target;
    }

    public boolean hostFileExists(String remoteFile) {
        checkNotNull(emptyToNull(remoteFile));
        OverthereFile file = getRemoteConnection().getFile(remoteFile);
        return file.exists();
    }


    public String readHostFile(final String remoteFile) {
        checkNotNull(emptyToNull(remoteFile));
        OverthereFile file = getRemoteConnection().getFile(remoteFile);
        checkArgument(file.exists(), "File %s does not exist on host %s", remoteFile, getContainer().getManagingContainer().getHost());
        try {
            InputStream in = file.getInputStream();
            try {
                byte[] bytes = ByteStreams.toByteArray(in);
                return new String(bytes);
            } finally {
                closeQuietly(in);
            }
        } catch (IOException e) {
            throw new RuntimeIOException("Failed to read file " + remoteFile, e);
        }
    }

    public String[] readHostFileLines(final String remoteFile) {
        String data = readHostFile(remoteFile);
        Iterable<String> iterable = Splitter.on(getContainer().getManagingContainer().getHost().getOs().getLineSeparator()).split(data);
        return Iterables.toArray(iterable, String.class);
    }
}

==> jbossdm-plugin-3.9.1-sources.jar: com/xebialabs/deployit/plugin/jbossdm/step/BaseStep.java <==
package com.xebialabs.deployit.plugin.jbossdm.step;

import java.io.*;
import java.net.URL;
import java.util.List;
import java.util.Map;

import javax.script.*;

import org.python.core.Options;
import org.python.core.PyException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.base.Charsets;
import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import com.google.common.collect.Iterables;
import com.google.common.io.ByteStreams;
import com.google.common.io.Resources;

import com.xebialabs.deployit.plugin.api.flow.ExecutionContext;
import com.xebialabs.deployit.plugin.api.flow.Step;
import com.xebialabs.deployit.plugin.api.flow.StepExitCode;
import com.xebialabs.deployit.plugin.jbossdm.container.CliManagedContainer;
import com.xebialabs.deployit.plugin.jbossdm.exception.CliScriptException;
import com.xebialabs.overthere.OverthereConnection;
import com.xebialabs.overthere.OverthereFile;
import com.xebialabs.overthere.RuntimeIOException;
import com.xebialabs.overthere.local.LocalConnection;
import com.xebialabs.overthere.local.LocalFile;
import com.xebialabs.overthere.util.OverthereUtils;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.base.Strings.emptyToNull;
import static com.google.common.collect.Lists.newArrayList;
import static com.google.common.io.Closeables.closeQuietly;

@SuppressWarnings("serial")
public abstract class BaseStep implements Step {

    public static final String JYTHON_SCRIPT_ENGINE = "jython";

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    private final String script;
    private final int order;
    private final Map<String, Object> pythonContext;
    private String description;
    private CliManagedContainer container;
    private List<String> additionalLibraries = newArrayList();

    private String remoteWorkingDirPath;
    private boolean retainRemoteWorkingDirOnCompletion;

    private transient OverthereConnection localConn;
    private transient OverthereConnection remoteConn;
    private transient ExecutionContext ctx;
    private transient OverthereFile remoteWorkingDir;


    protected BaseStep(String script, int order, Map<String, Object> pythonContext, String description, CliManagedContainer container) {
        this.script = script;
        this.order = order;
        this.pythonContext = pythonContext;
        this.description = description;
        this.container = container;

    }

    public int getOrder() {
        return order;
    }

    @Override
    public String getDescription() {
        return description;
    }

    public StepExitCode handleExecute(ExecutionContext ctx) throws Exception {
        try {
            this.ctx = ctx;
            ScriptEngine scriptEngine = loadScriptEngine();
            executeScript(scriptEngine);
            return StepExitCode.SUCCESS;
        } finally {
            disconnect();
        }
    }

    protected void executeScript(ScriptEngine scriptEngine) throws IOException {
        Bindings bindings = createBindings(pythonContext);
        bindings.put("step", this);
        loadLibraryScriptsAndEval(script, scriptEngine, bindings);
    }

    protected void loadLibraryScriptsAndEval(String scriptPath, ScriptEngine scriptEngine, Bindings localBindings) throws IOException {
        String script = Resources.toString(Resources.getResource(scriptPath), Charsets.UTF_8);
        //need to do this in order to work around a bug in the python script engine, which does not use the bindings
        //to compile the script when using the method scriptEngine.eval(String,Bindings).
        Bindings origEngineBindings = scriptEngine.getBindings(ScriptContext.ENGINE_SCOPE);
        try {
            Bindings engineAndLocalScope = new SimpleBindings();
            engineAndLocalScope.putAll(origEngineBindings);
            engineAndLocalScope.putAll(localBindings);
            scriptEngine.setBindings(engineAndLocalScope, ScriptContext.ENGINE_SCOPE);
            loadLibraryScripts(additionalLibraries, scriptEngine);
            logger.debug("Executing script " + scriptPath);
            if (logger.isTraceEnabled()) {
                logger.trace(script);
            }
            scriptEngine.eval(script);
        } catch (ScriptException e) {
            throwCliScriptException(scriptPath, e);
        } finally {
            scriptEngine.setBindings(origEngineBindings, ScriptContext.ENGINE_SCOPE);
        }
    }

    protected ScriptEngine loadScriptEngine() throws IOException {
        Options.includeJavaStackInExceptions = false;
        ScriptEngine scriptEngine = new ScriptEngineManager().getEngineByName(JYTHON_SCRIPT_ENGINE);
        checkNotNull(scriptEngine, "Jython Script Engine cannot be initialized. Make sure jython jars are on the class path.");
        loadLibraryScripts(container.getManagingContainer().getLibraries(), scriptEngine);
        return scriptEngine;
    }

    protected void loadLibraryScripts(List<String> libs, ScriptEngine scriptEngine) throws IOException {
        if (!libs.isEmpty()) {
            for (String library : libs) {
                String script = Resources.toString(Resources.getResource(library), Charsets.UTF_8);
                checkNotNull(script, "Library {} cannot be found on class path.", library);
                try {
                    scriptEngine.eval(script);
                } catch (ScriptException e) {
                    throwCliScriptException(library, e);
                }
            }
        }
    }

    protected void throwCliScriptException(String script, Throwable e) {
        while(e.getCause() != null) {
            if(e instanceof ScriptException) {
                e = e.getCause();
            } else if(e.getClass() == PyException.class && e.getMessage() == null) {
                e = e.getCause();
            } else {
                break;
            }
        }
        throw new CliScriptException(script, e);
    }

    protected Bindings createBindings(Map<String, Object> variables) {
        Bindings bindings = new SimpleBindings();
        bindings.putAll(variables);
        return bindings;
    }

    public Object executeCliCommand(String cmd) {
        return container.getManagingContainer().execute(ctx, cmd);
    }

    public ExecutionContext getCtx() {
        return ctx;
    }

    public void setAdditionalLibraries(List<String> additionalLibraries) {
        this.additionalLibraries = additionalLibraries;
    }

    protected List<String> getAdditionalLibraries() {
        return additionalLibraries;
    }

    public CliManagedContainer getContainer() {
        return container;
    }

    public OverthereFile getRemoteWorkingDirectory() {
        if (remoteWorkingDir == null) {
            OverthereFile tempDir;
            if (Strings.isNullOrEmpty(getRemoteWorkingDirPath())) {
                tempDir = getRemoteConnection().getTempFile("jbossdm_plugin", ".tmp");
            } else {
                tempDir = getRemoteConnection().getFile(getRemoteWorkingDirPath());
            }
            tempDir.mkdir();
            remoteWorkingDir = tempDir;
        }
        return remoteWorkingDir;
    }


    public OverthereConnection getLocalConnection() {
        if (localConn == null) {
            localConn = LocalConnection.getLocalConnection();
        }
        return localConn;
    }

    public OverthereConnection getRemoteConnection() {
        if (remoteConn == null) {
            remoteConn = container.getManagingContainer().getHost().getConnection();
        }
        return remoteConn;
    }

    protected void disconnect() {
        if (localConn != null)
            closeQuietly(localConn);

        if (!Strings.isNullOrEmpty(getRemoteWorkingDirPath()) && !isRetainRemoteWorkingDirOnCompletion()) {
            getRemoteWorkingDirectory().deleteRecursively();
        }

        if (remoteConn != null)
            closeQuietly(remoteConn);

        remoteWorkingDir = null;
        localConn = null;
        remoteConn = null;
    }

    public String getRemoteWorkingDirPath() {
        return remoteWorkingDirPath;
    }

    public void setRemoteWorkingDirPath(String remoteWorkingDirPath) {
        this.remoteWorkingDirPath = remoteWorkingDirPath;
    }

    public boolean isRetainRemoteWorkingDirOnCompletion() {
        return retainRemoteWorkingDirOnCompletion;
    }

    public void setRetainRemoteWorkingDirOnCompletion(boolean deleteWorkingDirOnCompletion) {
        this.retainRemoteWorkingDirOnCompletion = deleteWorkingDirOnCompletion;
    }

    public OverthereFile uploadToWorkingDirectory(String content, String fileName) {
        getCtx().logOutput("Uploading file " + fileName + " to working directory.");
        OverthereFile target = getRemoteWorkingDirectory().getFile(fileName);
        OverthereUtils.write(content.getBytes(), target);
        return target;
    }

    public OverthereFile uploadToWorkingDirectory(File content, String fileName) {
        return uploadToWorkingDirectory(LocalFile.valueOf(content), fileName);
    }

    public OverthereFile uploadToWorkingDirectory(OverthereFile content, String fileName) {
        String fileType = content.isDirectory() ? "directory" : "file";
        getCtx().logOutput("Uploading " + fileType + " " + fileName + " to working directory.");
        OverthereFile target = getRemoteWorkingDirectory().getFile(fileName);
        content.copyTo(target);
        return target;
    }

    public OverthereFile uploadToWorkingDirectory(URL content, String fileName) {
        getCtx().logOutput("Uploading file " + fileName + " to working directory.");
        OverthereFile target = getRemoteWorkingDirectory().getFile(fileName);
        OutputStream out = target.getOutputStream();
        try {
            Resources.copy(content, out);
        } catch (IOException e) {
            throw new RuntimeIOException(e);
        } finally {
            closeQuietly(out);
        }
        return target;
    }

    public boolean hostFileExists(String remoteFile) {
        checkNotNull(emptyToNull(remoteFile));
        OverthereFile file = getRemoteConnection().getFile(remoteFile);
        return file.exists();
    }


    public String readHostFile(final String remoteFile) {
        checkNotNull(emptyToNull(remoteFile));
        OverthereFile file = getRemoteConnection().getFile(remoteFile);
        checkArgument(file.exists(), "File %s does not exist on host %s", remoteFile, getContainer().getManagingContainer().getHost());
        try {
            InputStream in = file.getInputStream();
            try {
                byte[] bytes = ByteStreams.toByteArray(in);
                return new String(bytes);
            } finally {
                closeQuietly(in);
            }
        } catch (IOException e) {
            throw new RuntimeIOException("Failed to read file " + remoteFile, e);
        }
    }

    public String[] readHostFileLines(final String remoteFile) {
        String data = readHostFile(remoteFile);
        Iterable<String> iterable = Splitter.on(getContainer().getManagingContainer().getHost().getOs().getLineSeparator()).split(data);
        return Iterables.toArray(iterable, String.class);
    }
}
